# Viral production and burst size calculation
# Necessary data: viral abundance incubation time series (VLP) and host (HB, Syne, Proc, Euk) abundance (Host) at the beginning of incubation
# VLP is a 13*6 dataframe, with viral abundance of VLP (VLP1+VLP2) at day (column 1-3) and night (column 4-6) in three replicates of incubation
# Host is a 6*4 dataframe, with T0 host (HB) density (columns 1-2 = diluted and control at day; columns 3-4 = diluted and control at night)
VPBS=function(VLP,Host,Time){
library(spatialEco) # Package to find local maxima and minima Ver 1.3-2)
# 1. Viral production; calculated by (1) linear regression between viral abundance and time, (2) Winter et al. (2004) mean VP by the differences of local max and min
MVLP.DN<-cbind(VLP[,1:3],rowMeans(VLP[,1:3],na.rm=T),VLP[,4:6],rowMeans(VLP[,4:6],na.rm=T)) # Daytime (column 1:4, column 4 is mean VLP) and night time mean VLP (column 5:8, column 8 is mean VLP) 
colnames(MVLP.DN)<-c("D1","D2","D3","D","N1","N2","N3","N")
# Calculate the mean host (HB only) density at T0 (row 1:3) and T12 (row 4:6)
HostT0<-Host[1:3,]# Diluted host at day, control host at day, diluted host at night, control host at night 
MHost<-rbind(HostT0,colMeans(HostT0))
colnames(MHost)<-c("DDay","CDay","DNight","CNight")
VP.RM<-mat.or.vec(8,8) # Viral production by regression (VP, R2 and p value) and multiple peaks, ANCOVA test for the two regression, T-test for VP-L and VP-M
for (i in 1:2){ # Viral production (from mean VLP) at day and night 
  VLP.A<-MVLP.DN[,c(1+(i-1)*4):c(i*4)]
  # (1) linear regression to calculate viral production
  for (j in 1:4){
    VLP.T<-VLP.A[,j]
    R<-summary(lm(VLP.T~Time)) # Regression between VLP density and time
    VP.RM[c((i-1)*4+j),1]<-MHost[j,c(i*2)]/MHost[j,c(1+2*(i-1))]*R$coefficients[2,1] # considering dilution index (Bacteria before dilution/after dilution)
    VP.RM[c((i-1)*4+j),2]<-R$r.squared # R square of regression
    VP.RM[c((i-1)*4+j),3]<-R$coefficients[2,4] # p value of regression
    # (2) Consider multiple peaks as Winter et al. (2004)
    MinMax<-local.min.max(VLP.T) # find the local minima and maxima
    DataMin<-as.matrix(mat.or.vec(2,length(MinMax$minima)))
    for (m in 1:length(MinMax$minima)){
      DataMin[,m]<-c(Time[VLP.T==MinMax$minima[m]],VLP.T[VLP.T==MinMax$minima[m]])
    }
    DataMax<-as.matrix(mat.or.vec(2,length(MinMax$maxima)))
    for (m in 1:length(MinMax$maxima)){
      DataMax[,m]<-c(Time[VLP.T==MinMax$maxima[m]],VLP.T[VLP.T==MinMax$maxima[m]])
    }
    if(DataMax[1,1]==0){ # Decrease of viruses in the beginning of incubation
      VP.RM[c((i-1)*4+j),4]=(MHost[j,c(i*2)]/MHost[j,c(1+2*(i-1))])*mean((DataMax[2,2:c(length(DataMax)/2)]-DataMin[2,])/(DataMax[1,2:c(length(DataMax)/2)]-DataMin[1,]))
    }else{ # Increase of viruses in the beginning of incubation
      DataMin<-cbind(matrix(c(Time[1],VLP.T[1]),nrow=2),DataMin)
      VP.RM[c((i-1)*4+j),4]=(MHost[j,c(i*2)]/MHost[j,c(1+2*(i-1))])*mean((DataMax[2,]-DataMin[2,])/(DataMax[1,]-DataMin[1,]))
    }
  }
}

# ANCOVA test for day/night effect on regression
for (i in 1:4){
  VLP.Data<-as.data.frame(rbind(cbind(MVLP.DN[,i],matrix(0,nrow=length(Time),ncol=1),Time),cbind(MVLP.DN[,c(4+i)],matrix(1,nrow=length(Time),ncol=1),Time))) # Compare day (type=0) and night (type=1) regressions 
  colnames(VLP.Data)<-c("VLP","Type","Time")
  R1<-aov(VLP~Time+Type,data=VLP.Data)
  R2<-aov(VLP~Time*Type,data=VLP.Data)
  Test<-anova(R1,R2)
  VP.RM[i,5:6]<-c(Test[2,5],Test[2,6]) # The F value (row 1) and p value (row 2) of ANCOVA
}
# t-test for day/night effect on VP-L and VP-M
Ttest<-mat.or.vec(2,2) # Row 1: VP-L, Row 2: VP-M
VP<-mat.or.vec(6,3)
VP[,2:3]<-VP.RM[c(1:3,5:7),c(1,4)]
VP[1:3,1]<-c(1)
VP[4:6,1]<-c(2)
colnames(VP)<-c("Group","VP_L","VP_M")
VP<-as.data.frame(VP)
for (i in 1:2){
  Test<-t.test(VP[,i+1]~VP[,1])
  Ttest[i,1]<-Test$statistic # t-value
  Ttest[i,2]<-Test$p.value # p value
}
VP.RM[1:2,7:8]<-Ttest
colnames(VP.RM)<-c("VP_L","R2","p","VP_M","ANCOVA_F","ANCOVA_p","Ttest-t","Ttest-p")
rownames(VP.RM)<-c("Day1","Day2","Day3","Day","Night1","Night2","Night3","Night")

# 2. Calculate the hosts lyzed by viruses in one hour and the % of bacteria lyzed in one hour
# Use mean burst size 24 in marine systems (Parada et al. 2006)
Bac.M<-VP.RM[,c(1,4)]/24 # Bacterial mortality by viral lysis (# bacteria per hour)
Lys.RR<-100*as.matrix(Bac.M[,1]/c(MHost[1:4,2],MHost[1:4,4])) # % of environmental bacteria lysed per hour (VP by regression)
Lys.MR<-100*as.matrix(Bac.M[,2]/c(MHost[1:4,2],MHost[1:4,4])) # % of environmental bacteria lysed per hour (VP by local mean)
Lysis<-cbind(Bac.M,Lys.RR,Lys.MR)
colnames(Lysis)<-c("Mor.L","Mor.M","Lys.L","Lys.M") # mortality and ratio of bacteria killed by viral lysis calculated from regression (col 1,3) and multiple peak (col 2,4) 

Results<-list(VP.Stat=VP.RM,Bac.Mor=Lysis)
return(Results)
}